package org.artifactory.addon.bridge;

import com.jfrog.bh.acl.consumer.bridge.BundleArtifactoryConsumerBridge;
import com.jfrog.bh.acl.consumer.model.BundleArtifact;
import com.jfrog.bh.acl.consumer.model.BundleArtifactoryVersion;
import com.jfrog.bh.acl.consumer.model.BundleRepoConfig;
import com.jfrog.bh.acl.consumer.model.BundleSourceItem;
import com.jfrog.bh.acl.consumer.model.build.BundleSourceBuild;
import com.jfrog.bh.acl.error.base.BundleException;
import com.jfrog.sysconf.SysConfig;
import lombok.NonNull;
import org.jfrog.access.client.AccessClient;
import org.jfrog.access.common.PermissionScopeResourceType;
import org.jfrog.jfevent.event.NotificationEvent;
import org.jfrog.jfevent.model.payload.NotificationPayload;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.springframework.stereotype.Component;

import javax.annotation.Nullable;
import java.io.InputStream;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Supplier;

/**
 * Empty implementation of the {@link BundleArtifactoryConsumerBridge} for Artifactory OSS.
 * <p>
 * The non-empty implementation resides in the Artifactory Pro, and has a higher priority
 * than this bean (in Artifactory Pro deployments this empty implementation is suppressed).
 */
@Component
public class BundleArtifactoryConsumerBridgeEmptyImpl implements BundleArtifactoryConsumerBridge {

    private static final UnsupportedOperationException UNSUPPORTED_OPERATION =
            new UnsupportedOperationException("The 'Bundles' add-on is available in Artifactory Pro");

    @NonNull
    @Override
    public BundleArtifactoryVersion getRunningVersion() {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public SysConfig getSysConfig() {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public JdbcHelper getJdbcHelper() {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public DbType getDatabaseType() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public long getNextDatabaseId() {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public AccessClient getAccessClient() {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public BundleRepoConfig getRepositoryConfig(@NonNull String repoKey) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public boolean repositoryPathExists(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public boolean hasPermission(@NonNull PermissionScopeResourceType type,
            @NonNull String permission, @NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public boolean bundleRepositoryExists(@NonNull String repoKey, @Nullable String projectKey) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void createBundleRepository(@NonNull String repoKey, @Nullable String projectKey) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public Set<BundleSourceItem> collectBundleSourceItemsByAql(@NonNull String aql) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public Set<BundleSourceItem> collectBundleSourceItemsByBuild(
            @NonNull String buildName, @NonNull String buildNumber,
            @Nullable String buildRepo, @Nullable String buildStarted) throws BundleException {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public Set<BundleSourceItem> collectBundleSourceItemsByFiles(@NonNull Set<String> files) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void copyFile(@NonNull String repoPathFrom, @NonNull String repoPathTo) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public InputStream readFile(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void storeFile(@NonNull String repoPath, byte[] payload) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void deleteFile(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void deleteFiles(@NonNull Set<String> reposPaths) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public BundleArtifact getFileInfo(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public List<BundleArtifact> getChildFileInfos(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public Map<String, Collection<String>> getProperties(@NonNull String repoPath) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void addPropertiesRecursively(@NonNull String repoPath, @NonNull Map<String, List<String>> properties) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public BundleSourceBuild getBuild(
            @NonNull String buildName, @NonNull String buildNumber,
            @Nullable String buildRepo, @Nullable String buildStarted) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public Optional<String> getBuildProperty(
            @NonNull String buildPropertyName,
            @NonNull String buildName, @NonNull String buildNumber,
            @Nullable String buildRepo, @Nullable String buildStarted) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public String scheduleBundleCleanup(@NonNull String cronExpression) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void deScheduleBundleCleanup(@NonNull String taskToken) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void processAsync(
            @NonNull Supplier<? extends NotificationEvent<? extends NotificationPayload>> eventSupplier) {
        throw UNSUPPORTED_OPERATION;
    }

    @NonNull
    @Override
    public String getArtifactoryUrl() {
        throw UNSUPPORTED_OPERATION;
    }
}
