package org.artifactory.build;

import org.apache.commons.io.IOUtils;
import org.artifactory.api.build.request.BuildArtifactoryRequest;
import org.artifactory.repo.RepoPath;
import org.artifactory.webapp.servlet.HttpArtifactoryRequest;
import org.jetbrains.annotations.NotNull;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Enumeration;

import static org.artifactory.checksum.ChecksumInfo.TRUSTED_FILE_MARKER;

/**
 * BE EXTREMELY CAREFUL
 * It is a necessity of the build-info repo to buffer the entire content of a build json - whatever {@param httpRequest}
 * you initialize this class with will be read into memory in its entirety!
 *
 * @author Yuval Reches
 * @author Gidi Shabat
 */
public class BuildArtifactoryRequestImpl extends HttpArtifactoryRequest implements BuildArtifactoryRequest {

    private byte[] content;

    public BuildArtifactoryRequestImpl(HttpServletRequest httpRequest) throws IOException {
        super(httpRequest);
        content = IOUtils.toByteArray(httpRequest.getInputStream());
    }

    @Override
    public InputStream getInputStream() throws IOException {
        return new ByteArrayInputStream(content);
    }

    @Override
    public void replaceContent(byte[] newContent) {
        content = newContent.clone();
    }

    @Override
    public long getContentLength() {
        return content.length;
    }

    @Override
    public Enumeration<String> getHeaderValues(@NotNull String headerName) {
        switch (headerName) {
            case CHECKSUM_SHA1:
            case CHECKSUM_MD5:
            case CHECKSUM_SHA256:
                return Collections.enumeration(Collections.singletonList(TRUSTED_FILE_MARKER));
            default:
                return super.getHeaderValues(headerName);
        }
    }

    @Override
    public void setRepoPath(RepoPath repoPath) {
        super.setRepoPath(repoPath);
    }
}
