package org.artifactory.ph;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import org.artifactory.common.repo.PackageType;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.Repo;
import org.jetbrains.annotations.Nullable;

import java.util.EnumSet;

/**
 * Utility mapper between {@link org.artifactory.descriptor.repo.RepoType} to {@link PackageType}
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public final class PackageRepoTypeMapper {

    static final EnumSet<RepoType> EXCLUDED_REPO_TYPES = EnumSet.of(
            RepoType.Distribution, RepoType.ReleaseBundles, RepoType.BuildInfo, RepoType.PipeInfo, RepoType.Support);

    @Nullable
    static PackageType packageType(Repo<?, ?> repo) {
        return packageType(repo.getDescriptor().getType());
    }

    @Nullable
    static PackageType packageType(RepoType repoType) {
        if (EXCLUDED_REPO_TYPES.contains(repoType)) {
            return null;
        }
        return PackageType.valueOf(repoType.getType().toUpperCase());
    }
}
