package org.artifactory.ph;

import com.jfrog.ph.acl.exception.PackageException;
import lombok.NonNull;
import org.artifactory.common.repo.PackageType;

import javax.annotation.Nullable;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.Response;

/**
 * Provides the access to package-handler's PackageResource functionality from artifactory-oss/artifactory-pro.
 *
 * @author Volodymyr Shulga
 */
public interface PackageResourceService {

    /**
     * Executes reindex request on repository using package-handler functionality.
     * @param packageType   type of the package
     * @param repoKey       repository key
     * @return              response from reindex
     */
    Response reindex(@NonNull PackageType packageType, @NonNull String repoKey) throws PackageException;

    /**
     * @param packageType   like nuget/npm.
     * @param version       package api version if available (like v2/v3) or null.
     * @param repoKey       repository key.
     * @return sub resource for specified repoType/version from package-handler. Null if implementation not found.
     */
    @Nullable
    Object getVersionedSubResource(@NonNull PackageType packageType, String version, @NonNull String repoKey,
            HttpServletRequest request) throws PackageException;

    /**
     *
     * @param packageType   like nuget/npm.
     * @param context       uri path
     * @return              sub resource for specified repoType/context from package-handler.
     * Null if implementation not found.
     */
    @Nullable
    Object getNonVersionedSubResource(@NonNull PackageType packageType, @NonNull String context,
            HttpServletRequest request) throws PackageException;

    /**
     * The method is used by Artifactory UI in order to retrieve a package info.
     * @param repoKey   repository key.
     * @param path      package path.
     * @return          Response with package info for UI.
     */
    Response getPackageInfo(String repoKey, String path) throws PackageException;

    /**
     * Support displaying of a README file in the UI under the "packages" tab for a specific version.
     * @param fullRepoPath  from pattern <repoKey>/<pathToFile>
     * @return              The content of the README file if exists.
     */
    Response getVersionNativeReadme(String type, String fullRepoPath) throws PackageException;
}
