package org.artifactory.ph;

import com.jfrog.ph.acl.exception.PackageException;
import com.jfrog.ph.acl.service.PackageResourceServiceAdapter;
import com.jfrog.ph.acl.service.PackageSpecificationAdapter;
import lombok.NonNull;
import org.artifactory.common.repo.PackageType;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.Response;

/**
 * @author Volodymyr Shulga
 */
public class PackageResourceServiceImpl implements PackageResourceService {

    private final PackageResourceServiceAdapter packageResourceServiceAdapter;
    private final PackageSpecificationAdapter packageSpecificationAdapter;

    public PackageResourceServiceImpl(PackageResourceServiceAdapter packageResourceServiceAdapter,
            PackageSpecificationAdapter packageSpecificationAdapter) {
        this.packageResourceServiceAdapter = packageResourceServiceAdapter;
        this.packageSpecificationAdapter = packageSpecificationAdapter;
    }

    @Override
    public Response reindex(@NonNull PackageType packageType, @NonNull String repoKey)
            throws PackageException {
        if (!packageSpecificationAdapter.isSupportedPackage(packageType)) {
            return null;
        }
        return packageResourceServiceAdapter.reindex(packageType, repoKey);
    }

    @Override
    public Object getVersionedSubResource(@NonNull PackageType packageType, String version, @NonNull String repoKey,
            HttpServletRequest request)
            throws PackageException {
        if (!packageSpecificationAdapter.isSupportedPackage(packageType)) {
            return null;
        }
        return packageResourceServiceAdapter.getVersionedSubResource(packageType, version, repoKey, request);
    }

    @Override
    public Object getNonVersionedSubResource(@NonNull PackageType packageType, @NonNull String context,
            HttpServletRequest request) throws PackageException {
        if (!packageSpecificationAdapter.isSupportedPackage(packageType)) {
            return null;
        }
        return packageResourceServiceAdapter.getNonVersionedSubResource(packageType, context, request);
    }

    @Override
    public Response getPackageInfo(String repoKey, String path) throws PackageException {
        if (!packageSpecificationAdapter.isSupportedPackage(repoKey)) {
            return null;
        }
        return packageResourceServiceAdapter.getPackageInfo(repoKey, path);
    }

    @Override
    public Response getVersionNativeReadme(String type, String fullRepoPath) throws PackageException {
        return packageResourceServiceAdapter.getVersionNativeReadme(type, fullRepoPath);
    }
}
