package org.artifactory.ph;

import org.artifactory.common.resource.BaseResource;
import com.jfrog.ph.acl.exception.PackageException;
import com.jfrog.ph.acl.exception.PackageVirtualTraversalException;
import com.jfrog.ph.acl.virtual.resolution.PackageMetadataRow;
import com.jfrog.ph.acl.virtual.resolution.PackageMetadataVersionsSet;
import com.jfrog.ph.acl.virtual.resolution.PackageRepo;
import com.jfrog.ph.acl.virtual.resolution.PackageRequestContext;
import com.jfrog.ph.acl.virtual.resolution.PackageVirtualResolutionAdapter;

import javax.annotation.Nonnull;
import java.io.InputStream;
import java.util.LinkedHashSet;
import java.util.Map;

public class PackageVirtualResolutionServiceImpl implements PackageVirtualResolutionService<PackageMetadataRow> {
    private final PackageVirtualResolutionAdapter<PackageMetadataRow> packageHandlerVirtualService;

    public PackageVirtualResolutionServiceImpl(PackageVirtualResolutionAdapter<PackageMetadataRow>
            packageVirtualResolutionAdapter) {
        this.packageHandlerVirtualService = packageVirtualResolutionAdapter;
    }

    @Override
    public boolean shouldUsePackageVirtualResolution(String virtualRepoKey) {
        return packageHandlerVirtualService.shouldUsePackageVirtualResolution(virtualRepoKey);
    }

    @Override
    public BaseResource getItemInfoFromVirtual(PackageRepo virtualRepo, PackageRequestContext context)
            throws PackageVirtualTraversalException {
        try {
            return packageHandlerVirtualService.getItemInfoFromVirtual(virtualRepo, context);
        } catch (PackageException e) {
            throw new PackageVirtualTraversalException(e);
        }
    }

    @Nonnull
    @Override
    public Map<String, LinkedHashSet<PackageMetadataRow>> extract(BaseResource res, String virtualRepoKey,
            String originalPath, PackageRepo sourceRepo)
            throws PackageVirtualTraversalException {
        try {
            return packageHandlerVirtualService.extract(res, virtualRepoKey, originalPath, sourceRepo);
        } catch (PackageException e) {
            throw new PackageVirtualTraversalException(e);
        }
    }

    @Nonnull
    @Override
    public InputStream toInputStream(Map<String, PackageMetadataVersionsSet<PackageMetadataRow>> metadata,
            PackageRequestContext requestContext) throws PackageVirtualTraversalException {
        return packageHandlerVirtualService.toInputStream(metadata, requestContext);
    }
}

