package org.artifactory.repo.migration.mappers;

import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.RepositoryConfigSource;
import org.artifactory.model.VirtualRepoConfig;
import org.artifactory.model.VirtualRepositoryRepoTypeConfig;
import org.artifactory.repo.virtual.VirtualCacheConfig;

import java.util.List;
import java.util.Map;

public class VirtualRepoConfigMapper {


    public static VirtualRepoConfig mapVirtualRepoConfig(VirtualRepoConfig virtualRepoConfig,
            Map<String, Object> newValues, CentralConfigDescriptor descriptor) {
        RepoConfigMigrationPatchUtils.buildBaseConfig(virtualRepoConfig.getBaseConfig(), newValues, descriptor);
        buildRepoTypeConfig(virtualRepoConfig.getRepoTypeConfig(), newValues);
        buildPackageTypeConfig(virtualRepoConfig.getPackageTypeConfig(), virtualRepoConfig.getPackageType(),
                newValues);
        RepoConfigMigrationPatchUtils.buildSecurityConfig(virtualRepoConfig.getSecurityConfig(), newValues, descriptor);
        return virtualRepoConfig;
    }

    private static void buildRepoTypeConfig(VirtualRepositoryRepoTypeConfig virtualRepositoryRepoTypeConfig,
            Map<String, Object> newValues) {
        for (Map.Entry<String, Object> newValue : newValues.entrySet()) {
            try {
                if ("repositories".equals(newValue.getKey())) {
                    Map<String, Object> properties = (Map<String, Object>) newValue.getValue();
                    List<String> propertiesList =
                            properties.entrySet().stream()
                                    .sorted(Map.Entry.comparingByKey())
                                    .map(Map.Entry::getValue)
                                    .map(String.class::cast)
                                    .toList();
                    virtualRepositoryRepoTypeConfig.setRepositoryRefs(propertiesList);
                } else if ("defaultDeploymentRepo".equals(newValue.getKey())) {
                    virtualRepositoryRepoTypeConfig.setDefaultDeploymentRepoRef(String.valueOf(newValue.getValue()));
                } else if ("artifactoryRequestsCanRetrieveRemoteArtifacts".equals(newValue.getKey())) {
                    virtualRepositoryRepoTypeConfig.setArtifactoryRequestsCanRetrieveRemoteArtifacts(
                            ((Boolean) newValue.getValue()));
                }
                if ("virtualCacheConfig".equals(newValue.getKey())) {
                    Map<String, Object> virtualCacheConfigNewValues = (Map<String, Object>) newValue.getValue();
                    for (Map.Entry<String, Object> virtualCacheConfigNewValue :
                            virtualCacheConfigNewValues.entrySet()) {
                        if ("virtualRetrievalCachePeriodSecs".equals(virtualCacheConfigNewValue.getKey())) {
                            VirtualCacheConfig virtualCacheConfig = new VirtualCacheConfig();
                            virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(
                                    ((Integer) virtualCacheConfigNewValue.getValue()).longValue());
                            virtualRepositoryRepoTypeConfig.setVirtualCacheConfig(virtualCacheConfig);
                        }
                    }
                }
            } catch (ClassCastException e) {
                throw new IllegalArgumentException("No values under " + newValue.getKey());
            }
        }
    }

    private static void buildPackageTypeConfig(Map<String, String> packageTypeConfig,
            RepoType repoType, Map<String, Object> newValues) {
        for (Map.Entry<String, Object> newValue : newValues.entrySet()) {
            try {
                if (RepoType.Bower.equals(repoType)) {
                    RepoConfigMigrationPatchUtils.externalDependenciesToConfig(packageTypeConfig, newValues);
                } else if (RepoType.Npm.equals(repoType)) {
                    RepoConfigMigrationPatchUtils.externalDependenciesToConfig(packageTypeConfig, newValues);
                } else if (RepoType.Docker.equals(repoType)) {
                    if ("dockerApiVersion".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("dockerApiVersion",
                                        String.valueOf(newValue.getValue()));
                    } else if ("resolveDockerTagsByTimestamp".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("resolveDockerTagsByTimestamp",
                                        String.valueOf(newValue.getValue()));
                    }else if ("cachingLocalForeignLayersEnabled".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("cachingLocalForeignLayersEnabled",
                                        String.valueOf(newValue.getValue()));
                    }
                } else if (repoType.isMavenGroup()) {
                    if ("pomRepositoryReferencesCleanupPolicy".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("pomRepositoryReferencesCleanupPolicy",
                                        String.valueOf(newValue.getValue()));
                    } else if ("forceMavenAuthentication".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("forceMavenAuthentication",
                                        String.valueOf(newValue.getValue()));
                    } else if (RepoType.P2.equals(repoType) && "p2".equals(newValue.getKey())) {
                        Map<String, Object> p2Values = (Map<String, Object>) newValue.getValue();
                        Map<String, Object> urlsValues = (Map<String, Object>) p2Values.get("urls");
                        List<String> urls = urlsValues.values().stream().map(String.class::cast).toList();
                        RepositoryConfigSource.entityToFlatMapKeys(urls, "urls", packageTypeConfig);

                        if ("forceP2Authentication".equals(newValue.getKey())) {
                            packageTypeConfig
                                    .put("forceP2Authentication",
                                            String.valueOf(newValue.getValue()));
                        }
                    }
                } else if (RepoType.NuGet.equals(repoType)) {
                    if ("forceNugetAuthentication".equals(newValue.getKey())) {
                        packageTypeConfig
                                .put("forceNugetAuthentication",
                                        String.valueOf(newValue.getValue()));
                    }
                } else if (RepoType.Go.equals(repoType)) {
                    RepoConfigMigrationPatchUtils.externalDependenciesToConfig(packageTypeConfig, newValues);
                }
            } catch (ClassCastException e) {
                throw new IllegalArgumentException("No values under " + newValue.getKey());
            }
        }
    }
}
