package org.artifactory.storage.service;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.binstore.service.ArtifactoryBinaryService;
import org.artifactory.storage.binstore.service.cleanup.BinariesGarbageCollectorService;
import org.artifactory.storage.binstore.service.cleanup.GCProviderWrapper;
import org.artifactory.storage.binstore.service.cleanup.GarbageCollectorStrategy;
import org.artifactory.storage.db.DbService;
import org.artifactory.storage.db.binstore.service.cleanup.MinorGcCollector;
import org.artifactory.storage.db.binstore.service.cleanup.TrashAndBinariesGCProvider;
import org.jfrog.storage.DbType;
import org.jfrog.storage.binstore.ifc.GarbageCollectorInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author Uriah Levy
 */
@Service
public class BinariesGarbageCollectorServiceImpl implements BinariesGarbageCollectorService {

    private final RepositoryService repoService;
    private final TrashService trashService;
    private final ArtifactoryBinaryService binaryService;
    private final SecurityService securityService;
    private final DbType databaseType;

    @Autowired
    public BinariesGarbageCollectorServiceImpl(RepositoryService repoService, TrashService trashService,
            ArtifactoryBinaryService binaryService, SecurityService securityService, DbService dbService) {
        this.repoService = repoService;
        this.trashService = trashService;
        this.binaryService = binaryService;
        this.securityService = securityService;
        databaseType = dbService.getDatabaseType();
    }

    @Override
    public GarbageCollectorInfo startMinorGC() {
        GCProviderWrapper gcProvider =
                new GCProviderWrapper(new TrashAndBinariesGCProvider(repoService, trashService, binaryService));
        MinorGcCollector garbageCollector =
                new MinorGcCollector(gcProvider, securityService, binaryService, databaseType,
                        GarbageCollectorStrategy.TRASH_AND_BINARIES);
        return garbageCollector.call();
    }
}
