package org.artifactory.metadata.service;

import com.jfrog.ph.acl.exception.PackageException;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.storage.db.event.service.metadata.mapper.MetadataEntityMapper;
import org.artifactory.storage.fs.service.FileService;
import org.artifactory.storage.fs.service.PropertiesService;
import org.jfrog.metadata.client.model.MetadataVersionRepo;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.io.IOException;
import java.util.List;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;

class MetadataEntityFacadeTest {
    @Mock private FileService fileService;
    @Mock private PropertiesService propService;
    @Mock private RepositoryService repoService;
    @Mock private MetadataEntityMapper mapper;

    private MetadataEntityFacade metadataEntityFacade;
    private static final String REPO_KEY = "nuget-local";

    @BeforeEach
    public void setUp() throws PackageException, IOException {
        MockitoAnnotations.openMocks(this);
        this.metadataEntityFacade = new MetadataEntityFacade(List.of(), fileService, propService, repoService, mapper);
    }

    @ParameterizedTest
    @MethodSource("testCreatePathDeletedEntityProvider")
    void testCreatePathDeletedEntityProvider(String repoKey, String path) {
        RepoPath repoPath = RepoPathFactory.create(repoKey, path);
        MetadataVersionRepo pathDeletedEntity = metadataEntityFacade.createPathDeletedEntity(repoPath);

        assertEquals(repoKey, pathDeletedEntity.getName());
        assertEquals(path, pathDeletedEntity.getLeadFilePath());
    }

    static Stream<Arguments> testCreatePathDeletedEntityProvider() {
        return Stream.of(
                Arguments.of(REPO_KEY, ""),
                Arguments.of(REPO_KEY, "a/b/c/"),
                Arguments.of(REPO_KEY, "a/b/c/pkg.nupkg")
        );
    }
}