package org.artifactory.ph;

import org.apache.commons.lang3.mutable.MutableBoolean;
import com.jfrog.ph.acl.service.PackageTaskExecutorSupplier;
import org.artifactory.schedule.CachedThreadPoolTaskExecutor;
import org.artifactory.util.ThreadDumpUtils;
import org.awaitility.Awaitility;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.testng.Assert.assertTrue;

class PackageTaskExecutorSupplierImplTest {


    @Test
    void testGetDefaultTaskExecutor() {
        CachedThreadPoolTaskExecutor cachedThreadPoolTaskExecutor = Mockito.mock(CachedThreadPoolTaskExecutor.class);
        PackageTaskExecutorSupplier packageTaskExecutorSupplier =
                new PackageTaskExecutorSupplierImpl(cachedThreadPoolTaskExecutor);
        assertEquals(packageTaskExecutorSupplier.getDefaultTaskExecutor(), cachedThreadPoolTaskExecutor);
    }

    @Test
    public void testCreateSingleTaskExecutor() throws InterruptedException {
        CachedThreadPoolTaskExecutor cachedThreadPoolTaskExecutor = Mockito.mock(CachedThreadPoolTaskExecutor.class);
        PackageTaskExecutorSupplier packageTaskExecutorSupplier =
                new PackageTaskExecutorSupplierImpl(cachedThreadPoolTaskExecutor);
        ExecutorService executorService = packageTaskExecutorSupplier.createCustomTaskExecutor(1);
        MutableBoolean backgroundThreadActive = new MutableBoolean();
        ThreadDumpUtils dumper = ThreadDumpUtils.builder().count(1).build();

        try {
            executorService.submit(() -> doSomething(backgroundThreadActive));

            Awaitility.await().atMost(3, TimeUnit.SECONDS)
                    .untilAsserted(() -> assertTrue(backgroundThreadActive.getValue()));

            StringBuilder firstThreadDump = new StringBuilder();
            dumper.dumpThreads(firstThreadDump);
            // The thread is alive
            assertTrue(firstThreadDump.toString().contains("art-fixed"));
        } finally {
            backgroundThreadActive.setValue(false);
            executorService.shutdownNow();
            executorService.awaitTermination(5, TimeUnit.SECONDS);
            assertTrue(executorService.isShutdown());
        }

    }

    private void doSomething(MutableBoolean backgroundThreadActive) {
        backgroundThreadActive.setValue(true);
        Awaitility.await().atMost(10, TimeUnit.SECONDS).until(backgroundThreadActive::isFalse);
    }
}