package org.artifactory.addon.analytics;

import org.artifactory.addon.Addon;
import org.artifactory.descriptor.repo.RepoType;

/**
 * Addon for collecting specific API interactions.
 * e.g. docker pull\push\login API calls.
 * The addon is currently used only by the onBoarding addon - but purpose to have more usages in the near future.
 *
 * @author BarakH
 */
public interface AnalyticsAddon extends Addon {

    /**
     * This method should be called when a successful package download process was accomplished.
     * e.g. when a docker manifest.json was downloaded.
     *
     * @param packageName - Full package name in case of a multipart name.
     */
    default void onSuccessfulPackageDownload(RepoType repoType, String repoKey, String packageName, String version) {
    }

    /**
     * This method should be called when a successful package download process was accomplished.
     * e.g. when a docker manifest.json was uploaded.
     *
     * @param packageName - Full package name in case of a multipart name.
     */
    default void onSuccessfulPackageUpload(RepoType repoType, String repoKey, String packageName, String version) {
    }

    /**
     * This method should be called when a user successfully generated a token.
     * e.g. when a user accomplished docker login process.
     */
    default void userCreatedToken(RepoType repoType, String userName) {
    }
}
