package org.artifactory.lifecycle.converter.postinit.v105;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.converter.ConverterPreconditionException;
import org.artifactory.converter.mirror.MetadataNegotiationService;
import org.artifactory.converter.postinit.PostInitConverter;
import org.artifactory.version.ArtifactoryServiceVersion;
import org.artifactory.version.CompoundVersionDetails;

/**
 * A negotiation converter for federation repositories.
 * Upon upgrade, triggers the negotiation process for federated repositories within {@link MetadataNegotiationService}
 * @author Alexei Vainshtein
 */
@Slf4j
public class FederationRepoNegotiationConverter extends PostInitConverter {

    public FederationRepoNegotiationConverter(ArtifactoryServiceVersion from, ArtifactoryServiceVersion until) {
        super(from, until);
    }

    @Override
    public void convert(CompoundVersionDetails source, CompoundVersionDetails target) {
        log.debug("Checking if federation repos need to perform negotiations.");
        ContextHelper.get().beanForType(MetadataNegotiationService.class)
                .negotiateUponUpgrade(source);
    }

    @Override
    public void assertConversionPrecondition(ArtifactoryHome home, CompoundVersionDetails fromVersion,
            CompoundVersionDetails toVersion) throws ConverterPreconditionException {
        if (!ArtifactoryHome.isBound()) {
            throw new ConverterPreconditionException("Artifactory home is missing");
        }
    }

    @Override
    public boolean isInterested(CompoundVersionDetails source, CompoundVersionDetails target) {
        return source != null && !source.isCurrent();
    }
}
