package org.artifactory.logging.layout.request;

import io.vavr.collection.Array;
import org.artifactory.logging.LoggingTestUtils;
import org.artifactory.logging.SaasLogNames;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.MDC;

import java.time.Instant;

/**
 * @author Tomer Slutsky
 */
public class RequestJsonLayoutTest {
    RequestJsonLayout requestJsonLayout = new RequestJsonLayout();

    @Test
    public void testDoLayout() {
        MDC.put("uber-trace-id", "traceid");
        String s = requestJsonLayout.doLayout(LoggingTestUtils
                .createEvent(Array.of(Instant.ofEpochMilli(System.currentTimeMillis()).toString(),
                                "traceid", "127.0.0.1", "", "HEAD", "http://localhost:8081", "200", 1996, 2010, 25,
                                "savior"),
                        RequestJsonLayoutTest.class));
        Assertions.assertTrue(s.contains("\"" + SaasLogNames.REQUEST_LOG.getLogKey() + "\":\"" +
                SaasLogNames.REQUEST_LOG.getLogName() + "\""));
        Assertions.assertTrue(s.contains("\"trace-id\":\"traceid\""));
        Assertions.assertTrue(s.contains("\"client\":\"127.0.0.1\""));
        Assertions.assertTrue(s.contains("\"useragent\":\"savior\""));
        Assertions.assertTrue(s.contains("\"path\":\"http://localhost:8081\""));
        Assertions.assertTrue(s.contains("\"user\":\"\""));
        Assertions.assertTrue(s.contains("\"method\":\"HEAD\""));
        Assertions.assertTrue(s.contains("\"request\":{"));
        Assertions.assertTrue(s.contains("\"response\":{"));
        Assertions.assertTrue(s.contains("\"datetime\":"));
        Assertions.assertTrue(s.contains("\"length\":2010"));
        Assertions.assertTrue(s.contains("\"length\":1996"));
        Assertions.assertTrue(s.contains("\"duration\":25"));
    }
}