package org.artifactory.logging.layout.service;

import ch.qos.logback.classic.spi.LoggingEvent;
import io.vavr.collection.Array;
import org.artifactory.logging.LoggingTestUtils;
import org.artifactory.logging.SaasLogNames;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.MDC;

/**
 * @author Tomer Slutsky
 */
public class ServiceJsonLayoutTest {

    ServiceJsonLayout serviceJsonLayout = new ServiceJsonLayout();

    @Test
    public void testDoLayout() {
        MDC.put("uber-trace-id", "traceid");
        LoggingEvent event = LoggingTestUtils
                .createEvent(Array.of(), ServiceJsonLayout.class);
        event.setCallerData(Array.of(new StackTraceElement(this.getClass().getName(),
                "testMethod", "ServiceJsonLayoutTest.java", 1)).asJava().toArray(new StackTraceElement[0]));
        String s = serviceJsonLayout.doLayout(event);
        Assertions.assertTrue(s.contains("\"" + SaasLogNames.SERVICE_LOG.getLogKey() + "\":\"" +
                SaasLogNames.SERVICE_LOG.getLogName() + "\""));
        Assertions.assertTrue(s.contains("\"trace_id\":\"traceid\""));
        Assertions.assertTrue(s.contains("\"service\":\"jfrt\""));
        Assertions.assertTrue(s.contains("\"loglevel\":\"INFO\""));
        Assertions.assertTrue(s.contains("\"class\":\"ServiceJsonLayoutTest.java\""));
        Assertions.assertTrue(s.contains("\"line\":\"1\""));
        Assertions.assertTrue(s.contains("\"thread\":\"main\""));
        Assertions.assertTrue(s.contains("\"app\":{"));
        Assertions.assertTrue(s.contains("\"message\":\"\""));
        Assertions.assertTrue(s.contains("\"datetime\":"));
    }
}