package org.artifactory.model;

import org.artifactory.descriptor.repo.DockerApiVersion;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.TerraformType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;

import java.util.Map;

import static org.artifactory.util.PackageTypeConfigFields.BLOCK_PUSHING_SCHEMA_1;
import static org.artifactory.util.PackageTypeConfigFields.DOCKER_API_VERSION;
import static org.artifactory.util.PackageTypeConfigFields.DOCKER_TAG_RETENTION;
import static org.artifactory.util.PackageTypeConfigFields.SUPPRESS_POM_CONSISTENCY_CHECKS;
import static org.artifactory.util.PackageTypeConfigFields.TERRAFORM_TYPE;


/**
 * @author Mor Merhav
 */
class LocalRepoConfigTest {

    @ParameterizedTest
    @EnumSource(RepoType.class)
    void mavenGroupDefaultValues(RepoType repoType) {
        if (repoType.isMavenGroup()) {
            LocalRepoConfig localRepoConfig = LocalRepoConfig.create("key", repoType);

            Assertions.assertEquals(repoType, localRepoConfig.getPackageType());

            Map<String, String> packageTypeConfig = localRepoConfig.getPackageTypeConfig();
            Assertions.assertEquals(1, packageTypeConfig.size());
            Assertions.assertTrue(packageTypeConfig.containsKey(SUPPRESS_POM_CONSISTENCY_CHECKS.getKey()));

            String suppressPomConsistencyChecks = packageTypeConfig.get(SUPPRESS_POM_CONSISTENCY_CHECKS.getKey());
            Assertions.assertTrue(Boolean.parseBoolean(suppressPomConsistencyChecks));
        }
    }

    @Test
    void dockerDefaultValues() {
        LocalRepoConfig localRepoConfig = LocalRepoConfig.create("key", RepoType.Docker);

        Assertions.assertEquals(RepoType.Docker, localRepoConfig.getPackageType());

        Map<String, String> packageTypeConfig = localRepoConfig.getPackageTypeConfig();
        Assertions.assertEquals(3, packageTypeConfig.size());
        Assertions.assertTrue(packageTypeConfig.containsKey(DOCKER_API_VERSION.getKey()));
        Assertions.assertTrue(packageTypeConfig.containsKey(DOCKER_TAG_RETENTION.getKey()));
        Assertions.assertTrue(packageTypeConfig.containsKey(BLOCK_PUSHING_SCHEMA_1.getKey()));

        String dockerApiVersion = packageTypeConfig.get(DOCKER_API_VERSION.getKey());
        String dockerTagRetention = packageTypeConfig.get(DOCKER_TAG_RETENTION.getKey());
        String blockPushingSchema1 = packageTypeConfig.get(BLOCK_PUSHING_SCHEMA_1.getKey());

        Assertions.assertEquals(DockerApiVersion.V2.name(), dockerApiVersion);
        Assertions.assertEquals("1", dockerTagRetention);
        Assertions.assertTrue(Boolean.parseBoolean(blockPushingSchema1));
    }

    @Test
    void terraformDefaultValues() {
        LocalRepoConfig localRepoConfig = LocalRepoConfig.create("key", RepoType.Terraform);

        Assertions.assertEquals(RepoType.Terraform, localRepoConfig.getPackageType());

        Map<String, String> packageTypeConfig = localRepoConfig.getPackageTypeConfig();
        Assertions.assertEquals(1, packageTypeConfig.size());
        Assertions.assertTrue(packageTypeConfig.containsKey(TERRAFORM_TYPE.getKey()));

        String terraformType = packageTypeConfig.get(TERRAFORM_TYPE.getKey());
        Assertions.assertEquals(TerraformType.MODULE.getName(), terraformType);
    }
}