package org.artifactory.storage.db.build.itest.artifactbuilds;

import org.artifactory.aql.result.rows.AqlBuild;
import org.artifactory.storage.db.build.entity.BuildModule;
import org.artifactory.storage.db.fs.entity.Node;
import org.artifactory.storage.db.fs.entity.NodeBuilder;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.List;
import java.util.stream.Collectors;

public class ArtifactBuildsTest extends ArtifactBuildsTestBase {

    Node producedArtifact = new NodeBuilder()
            .nodeId(1)
            .file(true)
            .repo("test-repo")
            .path("a")
            .name("file1")
            .md5Actual("c78996a9838d3ce72bb380291645cee6")
            .sha1Actual("09ce045cb586ff4c2b5a2427b811823214daa379")
            .sha2("b700f353b3807d7abba3ed92cc08b626bfbcc5e8d465c2396c889d8f")
            .build();

    Node dependantArtifact = new NodeBuilder()
            .nodeId(2)
            .file(true)
            .repo("test-repo")
            .path("b")
            .name("file2")
            .md5Actual("a28996a9838d3ce72bb380291645cee6")
            .sha1Actual("08ce045cb586ff4c2b5a2427b811823214daa379")
            .sha2("5c1ecb23f747a080b16a89f2a197b64047ea8e5801f34b91e859087d")
            .build();

    @Test
    void testGetArtifactBuilds() throws SQLException {
        String PRODUCED_BUILD_NAME = "prod-build1";
        String USED_BUILD_NAME = "used-build2";

        createArtifact(producedArtifact);
        createArtifact(dependantArtifact);

        createBuild(1, PRODUCED_BUILD_NAME);
        createBuild(2, USED_BUILD_NAME);

        BuildModule produced = new BuildModule(1, 1, "produced");
        createBuildModule(produced);
        BuildModule used = new BuildModule(2, 2, "used");
        createBuildModule(used);


        linkModuleToArtifact(produced, producedArtifact);
        linkModuleToDependantArtifact(1, used, dependantArtifact);

        List<AqlBuild> prodBuilds =
                buildsAql.getProducedBuilds(producedArtifact.getSha1Actual(), producedArtifact.getMd5Actual())
                        .collect(Collectors.toList());

        Assert.assertEquals(prodBuilds.size(), 1);
        Assert.assertEquals(prodBuilds.get(0).getBuildName(), PRODUCED_BUILD_NAME);

        List<AqlBuild> dependant =
                buildsAql.getDependantBuilds(dependantArtifact.getSha1Actual(), dependantArtifact.getMd5Actual())
                        .collect(Collectors.toList());
        Assert.assertEquals(dependant.size(), 1);
        Assert.assertEquals(dependant.get(0).getBuildName(), USED_BUILD_NAME);

    }
}
