package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.jfrog.storage.StringInputStream;
import org.jfrog.storage.util.DbUtils;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.SQLException;

import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v143;
import static org.testng.Assert.assertTrue;

/**
 * @author Hussein Alshqerat
 */
public class V143CreateGinIndexOnNodeNameTest extends UpgradeBaseTest {
    private static final String CREATE_EXTENSION_QUERY = "CREATE EXTENSION IF NOT EXISTS pg_trgm;";
    private static final String DROP_EXTENSION_QUERY = "DROP EXTENSION pg_trgm;";
    private static final String DROP_GIN_INDEX_QUERY = "DROP INDEX IF EXISTS nodes_lower_node_name_gin_idx;";

    @BeforeMethod
    public void setup() throws IOException, SQLException {
        if (dbProperties.isPostgres()) {
            executeSqlStream(new StringInputStream(CREATE_EXTENSION_QUERY));
        }
    }

    @AfterMethod
    public void cleanUp() throws IOException, SQLException {
        if (dbProperties.isPostgres()) {
            executeSqlStream(new StringInputStream(DROP_GIN_INDEX_QUERY));
            executeSqlStream(new StringInputStream(DROP_EXTENSION_QUERY));
        }
    }

    @Test
    public void testCreatingGinIndexOnLowerNodeName() throws SQLException {
        testOnlyWithPostgresql();
        try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                dbProperties.getDbType(), v143)) {
            v143.converter().convert(context);
        }
        assertTrue(DbUtils.indexExists(jdbcHelper, "nodes", "node_name", "nodes_lower_node_name_gin_idx",
                dbProperties.getDbType()));
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return v143;
    }
}