package org.artifactory.ui.rest.service.admin.security.ldap.ldapsettings;

import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.artifactory.api.common.BasicStatusHolder;
import org.artifactory.descriptor.security.ldap.LdapSetting;
import org.jdom2.Verifier;

import static org.artifactory.ui.utils.RequestUtils.PASSWORD_PLACEHOLDER;

/**
 * @author Dor Solomon
 */
@Slf4j
@UtilityClass
public class LdapSettingsValidator {

    public static final String INVALID_PASSWORD_MSG =
            "LDAP password needs to be re-entered to allow changes in LDAP settings";


    public static void validate(LdapSetting ldapSetting, BasicStatusHolder statusHolder, boolean update) {
        validateKey(ldapSetting, statusHolder);
        if (update) {
            validatePassword(ldapSetting, statusHolder);
        }
    }

    private static void validateKey(LdapSetting ldapSetting, BasicStatusHolder statusHolder) {
        String error = Verifier.checkXMLName(ldapSetting.getKey());
        if (StringUtils.isNotBlank(error)) {
            statusHolder.error(error, log);
        }
    }

    private static void validatePassword(LdapSetting ldapSetting, BasicStatusHolder statusHolder) {
        if (ldapSetting.getSearch() != null &&
                PASSWORD_PLACEHOLDER.equals(ldapSetting.getSearch().getManagerPassword())) {
            statusHolder.error(INVALID_PASSWORD_MSG, log);
        }
    }
}
