package org.artifactory.ui.rest.service.builds.buildsinfo;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.rest.build.ContinueBuildFilter;
import org.artifactory.api.rest.common.model.continues.ContinueResult;
import org.artifactory.api.rest.common.model.continues.util.Direction;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.builds.GeneralBuildInfo;
import org.artifactory.ui.rest.model.continuous.dtos.ContinueBuildDto;
import org.artifactory.ui.rest.model.continuous.translators.ContinueBuildTranslator;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Dvir Twina
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetAllBuildNamesPagedService implements RestService {

    private BuildService buildService;

    @Autowired
    public void setBuildService(BuildService buildService) {
        this.buildService = buildService;
    }

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        ContinueBuildDto continueBuildDto = new ContinueBuildDto(request,
                projectKey -> buildService.getBuildInfoRepoKeyByProject(projectKey));
        continueBuildDto.setOrderBy(ContinueBuildFilter.OrderBy.BUILD_NAME.name());
        continueBuildDto.setDirection(Direction.ASC);
        ContinueBuildFilter continueBuildFilter = ContinueBuildTranslator.toBuildFilter(continueBuildDto);

        Long totalRequiredBuilds = getTotalRequiredBuilds(continueBuildFilter);
        ContinueResult<String> cr = createBuildNamesContinueResult(continueBuildFilter);

        cr.setTotalRequiredDataAmount(totalRequiredBuilds);
        response.iModel(cr);
    }

    private ContinueResult<String> createBuildNamesContinueResult(ContinueBuildFilter continueBuildFilter) {
        List<String> builds = buildService.getDistinctBuildNamesPaging(continueBuildFilter);
        List<GeneralBuildInfo> generalBuildInfoList = builds.stream().map(this::getBuildGeneralInfo).toList();

        String nextContinueState = null;
        if (generalBuildInfoList.size() == continueBuildFilter.getLimit()) {
            GeneralBuildInfo last = generalBuildInfoList.get(generalBuildInfoList.size() - 1);
            nextContinueState = ContinueBuildTranslator.buildIdToBase64(last);
        }

        List<String> buildNames = generalBuildInfoList.stream().map(GeneralBuildInfo::getBuildName).toList();
        return new ContinueResult<>(nextContinueState, buildNames);
    }

    @Nullable
    private Long getTotalRequiredBuilds(ContinueBuildFilter continueBuildFilter) {
        Long totalRequiredBuilds = null;
        if (continueBuildFilter.getContinueBuildId() == null) {
            totalRequiredBuilds = (long) buildService.getDistinctBuildNamesCount(continueBuildFilter);
        }
        return totalRequiredBuilds;
    }

    private GeneralBuildInfo getBuildGeneralInfo(String buildName) {
        return GeneralBuildInfo.builder()
                .buildName(buildName)
                .time(-1L)
                .build();
    }
}
