package org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.fetch;

import org.artifactory.api.rest.common.model.continues.ContinueResult;
import org.artifactory.api.rest.common.model.continues.FetchFunction;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.TreeFilter;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.repo.RepositoryNode;
import org.testng.annotations.Test;

import java.util.List;

import static org.testng.Assert.assertEquals;

/**
 * @author Omri Ziv
 */
public class RootFetchByRepoKeyTest extends RootFetchStrategyTest {

    @Test
    public void testByKeyOnDefaultContext() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        RootFetchByRepoKey rootFetchByRepoKey = new RootFetchByRepoKey(treeFilter);
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByRepoKey.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 8);
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 5L);
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 10L);

        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(5).getFunction().apply(0, 20);
        assertEquals(continueResult1.getData().size(), 5);
        ContinueResult<RepositoryNode> continueResult2 = fetchFunctions.get(6).getFunction().apply(0, 20);
        assertEquals(continueResult2.getData().size(), 10);
    }

    @Test
    public void testByKeyOnProjectContext() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setProjectKey(PROJECT_A);
        RootFetchByRepoKey rootFetchByRepoKey = new RootFetchByRepoKey(treeFilter);
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByRepoKey.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 4);
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 7L);
        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 5);
        assertEquals(continueResult1.getData().size(), 5);
    }

}
