package org.artifactory.ui.rest.service.admin.configuration.repositories;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.pipelines.InternalPipelinesService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.repo.system.SystemRepos.BUILDINFO_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.EDGE_UPLOADS_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.IN_TRANSIT_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.LOG_SHIPPING_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.PIPEINFO_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.RELEASE_BUNDLE_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.SUPPORT_BUNDLE_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.TRASH_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.USAGE_REPO_KEY;
import static org.mockito.Mockito.anyInt;
import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class DeleteRepositoryConfigServiceTest {

    @DataProvider
    public static Object[][] systemReposProvider() {
        return new Object[][] {
                {USAGE_REPO_KEY},
                {LOG_SHIPPING_REPO_KEY},
                {BUILDINFO_REPO_KEY},
                {PIPEINFO_REPO_KEY},
                {TRASH_REPO_KEY},
                {SUPPORT_BUNDLE_REPO_KEY},
                {IN_TRANSIT_REPO_KEY},
                {EDGE_UPLOADS_REPO_KEY},
                {RELEASE_BUNDLE_REPO_KEY}
        };
    }

    @Test
    public void testExecuteDeletePipeInfoRepo() {
        BuildService buildService = mock(BuildService.class);
        InternalPipelinesService pipelinesService = mock(InternalPipelinesService.class);
        RepositoryConfigService repositoryConfigService = mock(RepositoryConfigService.class);
        RepositoryService repositoryService = mock(RepositoryService.class);
        DeleteRepositoryConfigService<?> deleteRepositoryConfigService =
                new DeleteRepositoryConfigService<>(repositoryService, repositoryConfigService, null, buildService,
                        pipelinesService);
        ArtifactoryRestRequest<?> request = mock(ArtifactoryRestRequest.class);
        when(request.getPathParamByKey("repoKey")).thenReturn("pipe-info");
        when(pipelinesService.isPipeInfoRepo("pipe-info")).thenReturn(true);
        ArtifactoryRestResponse<?> restResponse = mock(ArtifactoryRestResponse.class);
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            Object argument = invocationOnMock.getArguments()[0];
            Assert.assertTrue(argument instanceof String);
            Assert.assertEquals(argument, "Deleting a system repo 'pipe-info' is not allowed.");
            return restResponse;
        }).when(restResponse).error(anyString());
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            int argument = (int) invocationOnMock.getArguments()[0];
            Assert.assertEquals(argument, 403);
            return restResponse;
        }).when(restResponse).responseCode(anyInt());
        deleteRepositoryConfigService.execute(request, restResponse);
    }

    @Test(dataProvider = "systemReposProvider")
    public void testDeleteSystemRepo(String repoKey) {
        RepositoryConfigService repositoryConfigService = mock(RepositoryConfigService.class);
        BuildService buildService = mock(BuildService.class);
        RepositoryService repositoryService = mock(RepositoryService.class);
        DeleteRepositoryConfigService<?> deleteRepositoryConfigService =
                new DeleteRepositoryConfigService<>(repositoryService, repositoryConfigService, null, buildService,
                        null);
        ArtifactoryRestRequest<?> request = mock(ArtifactoryRestRequest.class);
        when(request.getPathParamByKey("repoKey")).thenReturn(repoKey);
        ArtifactoryRestResponse<?> restResponse = mock(ArtifactoryRestResponse.class);
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            Object argument = invocationOnMock.getArguments()[0];
            Assert.assertTrue(argument instanceof String);
            Assert.assertEquals(argument, "Deleting a system repo '" + repoKey + "' is not allowed.");
            return restResponse;
        }).when(restResponse).error(anyString());
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            int argument = (int) invocationOnMock.getArguments()[0];
            Assert.assertEquals(argument, 403);
            return restResponse;
        }).when(restResponse).responseCode(anyInt());
        deleteRepositoryConfigService.execute(request, restResponse);
    }
}