package org.artifactory.rest.filter;

import org.artifactory.util.BasicHttpUtils;
import org.jfrog.common.config.metadata.MetadataResponse;

import javax.ws.rs.HttpMethod;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerResponseContext;
import javax.ws.rs.container.ContainerResponseFilter;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.Arrays;
import java.util.List;

import static org.artifactory.api.rest.constant.RepositoriesRestConstants.CACHE_CONTROL_NO_STORE;

/**
 * A Response Filter that adds a "Cache-Control: no-store" header to metadata endpoints response
 * that filter by @MetadataResponse
 * See {@link org.jfrog.common.config.metadata.MetadataResponse}
 */

@Provider
public class MetadataResponseFilter implements ContainerResponseFilter {
    @Override
    public void filter(ContainerRequestContext requestContext, ContainerResponseContext responseContext)
            throws IOException {
        if (!shouldLookForMetadataResponseAnnotation(requestContext, responseContext)) {
            return;
        }
        List<Annotation> annotations = Arrays.stream(responseContext.getEntityAnnotations()).toList();
        if (annotations.stream()
                    .map(Annotation::annotationType)
                    .anyMatch(annotation -> annotation.isAssignableFrom(MetadataResponse.class))) {
            responseContext.getHeaders().add(HttpHeaders.CACHE_CONTROL, CACHE_CONTROL_NO_STORE);
        }
    }

    private boolean shouldLookForMetadataResponseAnnotation(ContainerRequestContext requestContext,
                    ContainerResponseContext responseContext) {
        return requestContext.getMethod().equals(HttpMethod.GET)
                && BasicHttpUtils.isInSuccessOrRedirectResponseCodeRange(responseContext.getStatus());
    }
}
