package org.artifactory.rest.filter;

import org.apache.http.HttpStatus;
import org.jfrog.common.config.metadata.MetadataResponse;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mockito;

import javax.ws.rs.HttpMethod;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerResponseContext;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.stream.Stream;

import static org.artifactory.api.rest.constant.RepositoriesRestConstants.CACHE_CONTROL_NO_STORE;

class MetadataResponseFilterTest {
    @ParameterizedTest
    @MethodSource("filterArgs")
    void testFilterNoStoreCacheAnnotation (String httpMethod,
            int status,
            Annotation[] entityAnnotation,
            boolean headerShouldAppear)
            throws IOException {
        ContainerRequestContext requestContext = Mockito.mock(ContainerRequestContext.class);
        ContainerResponseContext responseContext = Mockito.mock(ContainerResponseContext.class);
        MultivaluedMap<String, Object> headers = new MultivaluedHashMap<>();
        MetadataResponseFilter metadataResponseFilter = new MetadataResponseFilter();

        Mockito.when(responseContext.getHeaders()).thenReturn(headers);
        Mockito.when(requestContext.getMethod()).thenReturn(httpMethod);
        Mockito.when(responseContext.getEntityAnnotations()).thenReturn(entityAnnotation);
        Mockito.when(responseContext.getStatus()).thenReturn(status);

        metadataResponseFilter.filter(requestContext, responseContext);

        if (headerShouldAppear) {
            Assertions.assertNotNull(headers.get(HttpHeaders.CACHE_CONTROL));
            Assertions.assertFalse(headers.get(HttpHeaders.CACHE_CONTROL).isEmpty());
            Assertions.assertEquals(CACHE_CONTROL_NO_STORE, headers.get(HttpHeaders.CACHE_CONTROL).get(0));
        } else {
            Assertions.assertNull(headers.get(HttpHeaders.CACHE_CONTROL));
            Assertions.assertTrue(headers.isEmpty());
        }
    }

    private static Stream<Arguments> filterArgs() {
        MetadataResponse metadataResponse = new MetadataResponse() {
            @Override
            public Class<? extends Annotation> annotationType() {
                return MetadataResponse.class;
            }
        };
        //httpMethod, statusCode, entityAnnotation, headerShouldAppear
        return Stream.of(
                Arguments.of(HttpMethod.GET, HttpStatus.SC_OK, new Annotation[] {metadataResponse}, true),
                Arguments.of(HttpMethod.GET, HttpStatus.SC_BAD_REQUEST, new Annotation[] {metadataResponse}, false),
                Arguments.of(HttpMethod.GET, HttpStatus.SC_OK, new Annotation[] {}, false),
                Arguments.of(HttpMethod.POST, HttpStatus.SC_OK, new Annotation[] {}, false));
    }
}